@@ -112,6 +112,16 @@ plot.sobolTIIlo <- function(x, ylim = NULL, ...)
   }
 }
 
+ggplot.sobolTIIlo <- function(x, ylim = NULL, ...)
+{
+  if (is.null(ylim)){
+    ylim <- c(min(rbind(0,x$tii.scaled["min. c.i."])), max(x$tii.scaled["max. c.i."]))
+  }
+  if (!is.null(x$y)) {
+    nodeggplot(listx = list(x$tii.scaled), xname="",ylim = ylim)
+  }
+}
+
 
 plotFG.sobolTIIlo <- function (x) 
 {
